<?xml version="1.0" encoding="UTF-8"?>
<gui name="kde-nm-connection-editor"
     version="3"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd" >

  <MenuBar>
    <Menu name="file" >
      <Action name="import_vpn" />
      <Action name="export_vpn" />
    </Menu>

    <Menu name="connection">
    <text>Connection</text>
      <Action name="connect_connection" />
      <Action name="disconnect_connection" />
      <Action name="add_connection" />
      <Action name="edit_connection" />
      <Action name="delete_connection" />
    </Menu>
  </MenuBar>

  <ToolBar name="mainToolBar" >
    <text>Main Toolbar</text>
    <Action name="add_connection" />
    <Separator/>
    <Action name="connect_connection" />
    <Action name="disconnect_connection" />
    <Action name="edit_connection" />
    <Action name="delete_connection" />
  </ToolBar>
</gui>
